const n = ...;
type board = array [1..n] of 1..n;
var A: board;
procedure Permute(i: integer);
  (* Form all permutations of 1 through n, in lexicographic order, that
     have A[1..i-1] fixed, i.e., permute A[i..n] in every possible way.
     It is assumed that A[i..n] holds the lexicographically smallest 
     arrangement of A[i..n] to start with.  The ordering in A[ ] is
     restored upon procedure exit, which is required for correct
     functioning.  A[ ] should be initialized to 1,2,...,n before the
     external call to the procedure, which is Permute(1). *)
  var j: integer;
  begin
    if i < n
      then begin
             (* Form all permutations with successively higher values
                in A[i].  The first value is already in place. *)
             Permute(i+1);
             (* Successively swap in next higher value. *)
             for j := i+1 to n do
               begin
                 SWAP(A[i],A[j]);
                 (* A[i+1..n] is the lexicographically smallest way
                    of arranging the data currently in A[i+1..n]. *)
                 Permute(i+1)
                 (* Invariant: upon return, A is identical to what
                    it was immediately after the SWAP. *)
               end;
             (* Maintain invariant: do a circular shift left one place of
                A[i..n], which restores A to the way it was when called. *)
             ROTATE(i)
           end
      else EMIT(A) (* Announce new complete permutation. *)
  end; (* Permute *)
